@@ -61,8 +61,8 @@ public:
   std::map<unsigned, WPXBinaryData> m_vects;
   std::vector<CDRPage> m_pages;
   std::map<unsigned, CDRColor> m_documentPalette;
-  std::map<unsigned, WPXString> m_fonts;
-  std::map<unsigned, CDRText> m_texts;
+  std::map<unsigned, CDRFont> m_fonts;
+  std::map<unsigned, std::vector<CDRText> > m_texts;
 
   unsigned _getRGBColor(const CDRColor &color);
   unsigned getBMPColor(const CDRColor &color);
